/*
 *  file_provider_linux.c
 *
 *  Copyright 2014 Michael Zillgith
 *
 *  This file is part of libIEC61850.
 *
 *  libIEC61850 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libIEC61850 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with libIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  See COPYING file for the complete license text.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <dirent.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "filesystem.h"

#include "stack_config.h"

#ifndef CONFIG_VIRTUAL_FILESTORE_BASEPATH
#define CONFIG_VIRTUAL_FILESTORE_BASEPATH "./vmd-filestore/"
#endif

static char* fileBasePath = CONFIG_VIRTUAL_FILESTORE_BASEPATH;

FileHandle
FileSystem_openFile(char* fileName, bool readWrite)
{
    int pathLength = strlen(fileBasePath) + 1;

    if (fileName != NULL)
        pathLength += strlen(fileName);

    char* fullPath = alloca(pathLength);

    strcpy(fullPath, fileBasePath);

    if (fileName != NULL)
        strcat(fullPath, fileName);

    FileHandle newHandle = NULL;

    if (readWrite)
        newHandle = (FileHandle) fopen(fullPath, "w");
    else
        newHandle = (FileHandle) fopen(fullPath, "r");

    return newHandle;
}

int
FileSystem_readFile(FileHandle handle, uint8_t* buffer, int maxSize)
{
    return fread(buffer, maxSize, 1, (FILE*) handle);
}

void
FileSystem_closeFile(FileHandle handle)
{
    fclose((FILE*) handle);
}

uint32_t
FileSystem_getFileInfo(char* filename, uint64_t* lastModificationTimestamp)
{
    struct stat fileStats;

    int pathLength = strlen(fileBasePath) + 1;

    if (filename != NULL)
        pathLength += strlen(filename);

    char* fullPath = alloca(pathLength);

    strcpy(fullPath, fileBasePath);

    if (filename != NULL)
        strcat(fullPath, filename);

    stat(fullPath, &fileStats);

    if (lastModificationTimestamp != NULL)
        *lastModificationTimestamp = fileStats.st_mtim.tv_sec * 1000;

    return fileStats.st_size;
}

DirectoryHandle
FileSystem_openDirectory(char* directoryName)
{
    int pathLength = strlen(fileBasePath) + 1;

    if (directoryName != NULL)
        pathLength += strlen(directoryName);

    char* fullPath = alloca(pathLength);

    strcpy(fullPath, fileBasePath);

    if (directoryName != NULL)
        strcat(fullPath, directoryName);

    DIR* dirHandle = opendir(fullPath);

    return dirHandle;
}

char*
FileSystem_readDirectory(DirectoryHandle directory, bool* isDirectory)
{
    struct dirent *dir;

    dir = readdir(directory);

    if (dir != NULL) {
        if (dir->d_name[0] == '.')
            return FileSystem_readDirectory(directory, isDirectory);
        else {
            if (isDirectory != NULL) {
                if (dir->d_type == DT_DIR)
                    *isDirectory = true;
                else
                    *isDirectory = false;
            }

            return dir->d_name;
        }
    }
    else
        return NULL;
}

void
FileSystem_closeDirectory(DirectoryHandle directory)
{
    closedir((DIR*) directory);
}

#if 0
int
main(int argc, char** argv)
{
    DirectoryHandle directory = FileSystem_openDirectory("/");

    if (directory != NULL) {
        char* fileName = FileSystem_readDirectory(directory);


        while (fileName != NULL) {
            printf("FILE: (%s)\n", fileName);
            fileName = FileSystem_readDirectory(directory);
        }

        FileSystem_closeDirectory(directory);
    }
    else
        printf("Error opening directory!\n");
}
#endif
