/*
 *  iso_connection_parameters.c
 *
 *  IsoConnectionParameters abstract data type to represent the configurable parameters of the ISO protocol stack.
 *
 *  Copyright 2013, 2014 Michael Zillgith
 *
 *  This file is part of libIEC61850.
 *
 *  libIEC61850 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libIEC61850 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with libIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  See COPYING file for the complete license text.
 */

#include "libiec61850_platform_includes.h"

#include "stack_config.h"

#include "iso_connection_parameters.h"

#include "ber_encoder.h"

IsoConnectionParameters
IsoConnectionParameters_create()
{
    IsoConnectionParameters self = (IsoConnectionParameters) calloc(1, sizeof(struct sIsoConnectionParameters));

    return self;
}

void
IsoConnectionParameters_destroy(IsoConnectionParameters self)
{
    free(self);
}

void
IsoConnectionParameters_setAcseAuthenticationParameter(IsoConnectionParameters self,
        AcseAuthenticationParameter acseAuthParameter)
{
    self->acseAuthParameter = acseAuthParameter;
}

void
IsoConnectionParameters_setTcpParameters(IsoConnectionParameters self, char* hostname, int tcpPort)
{
    self->hostname = hostname;
    self->tcpPort = tcpPort;
}

void
IsoConnectionParameters_setRemoteApTitle(IsoConnectionParameters self, char* apTitle, int aeQualifier)
{
    if (apTitle == NULL)
        self->remoteApTitleLen = 0;
    else {
        self->remoteApTitleLen = BerEncoder_encodeOIDToBuffer(apTitle, self->remoteApTitle, 10);
        self->remoteAEQualifier = aeQualifier;
    }
}

void
IsoConnectionParameters_setRemoteAddresses(IsoConnectionParameters self, uint32_t pSelector, uint16_t sSelector, uint16_t tSelector)
{
    self->remotePSelector = pSelector;
    self->remoteSSelector = sSelector;
    self->remoteTSelector = tSelector;
}


void
IsoConnectionParameters_setLocalApTitle(IsoConnectionParameters self, char* apTitle, int aeQualifier)
{
    if (apTitle == NULL)
        self->localApTitleLen = 0;
    else {
        self->localApTitleLen = BerEncoder_encodeOIDToBuffer(apTitle, self->localApTitle, 10);
        self->localAEQualifier = aeQualifier;
    }
}

void
IsoConnectionParameters_setLocalAddresses(IsoConnectionParameters self, uint32_t pSelector, uint16_t sSelector, uint16_t tSelector)
{
    self->localPSelector = pSelector;
    self->localSSelector = sSelector;
    self->localTSelector = tSelector;
}
